/*******************************************************************************
 *	BPoll.cpp	File poll class
 *			T.Barnaby,	BEAM Ltd,	1/4/05
 *******************************************************************************
 */
#include	<stdlib.h>
#include	<unistd.h>
#include	<errno.h>
#include	<BPoll.h>

BPoll::BPoll(){
	ofdsNum = 0;
	ofds = 0;
	ofdsNext = 0;
}

BPoll::~BPoll(){
	if(ofds)
		free(ofds);
	ofds = 0;
	ofdsNum = 0;
	ofdsNext = 0;
}

void BPoll::append(int fd, int events){
	ofds = (PollFd*)realloc(ofds, (ofdsNum + 1) * sizeof(PollFd));
	ofds[ofdsNum].fd = fd;
	ofds[ofdsNum].events = events;
	ofds[ofdsNum].revents = 0;
	ofdsNum++;
}

void BPoll::delFd(int fd){
	int	n;
	
	for(n = 0; n < ofdsNum; n++){
		if(ofds[n].fd == fd){
			break;
		}
	}
	
	if((n + 1) < ofdsNum)
		memcpy(&ofds[n], &ofds[n + 1], (ofdsNum - (n + 1)) * sizeof(PollFd));
	
	if(n == ofdsNext)
		ofdsNext = 0;

	ofdsNum--;
	ofds = (PollFd*)realloc(ofds, ofdsNum * sizeof(PollFd));
}

int BPoll::nextFd(int i){
	i++;
	if(i >= ofdsNum)
		i = 0;
	return i;
}

int BPoll::getPollFdsNum(){
	return ofdsNum;
}

BPoll::PollFd* BPoll::getPollFds(){
	return ofds;
}

BError BPoll::doPoll(int& fd, int timeoutUs){
	BError		err;
	int		timeoutMs;
	int		r;
	int		i;
	
	if(timeoutUs >= 0)
		timeoutMs = timeoutUs / 1000;
	else
		timeoutMs = -1;

	r = poll(ofds, ofdsNum, timeoutMs);

	if(r > 0){
		// Get next available fd in round robin fashion
		for(i = ofdsNext; ;){
			if(ofds[i].revents){
				fd = ofds[i].fd;
				ofdsNext = nextFd(i);
				break;
			}
			
			i = nextFd(i);
			if(i == ofdsNext)
				break;
		}
	}
	else if(r < 0){
		err.set(-errno, strerror(errno));
		fd = -errno;
	}
	else if(r == 0){
		err.set(-ETIMEDOUT, "Timeout");
		fd = -ETIMEDOUT;
	}

	return err;
}

void BPoll::clear(){
	if(ofds)
		free(ofds);
	ofds = 0;
	ofdsNum = 0;
	ofdsNext = 0;
}
